@@ -16,6 +16,7 @@
 #include <string>
 #include <vector>
 #include <map>
+#include <memory>
 #include <cstring>
 #include <cstdlib>
 #include <cinttypes>
@@ -24,14 +25,19 @@
 #include "lcf/writer_lcf.h"
 #include "lcf/reader_xml.h"
 #include "lcf/writer_xml.h"
+#include "lcf/rpg/eventpagecondition.h"
+#include "lcf/rpg/trooppagecondition.h"
+#include "lcf/rpg/terrain.h"
 #include "lcf/rpg/equipment.h"
 #include "lcf/rpg/parameters.h"
 #include "lcf/rpg/eventcommand.h"
 #include "lcf/rpg/movecommand.h"
 #include "lcf/rpg/treemap.h"
 #include "lcf/rpg/rect.h"
+#include "lcf/rpg/savepicture.h"
 #include "lcf/rpg/terms.h"
-#include "log.h"
+#include "lcf/rpg/saveeasyrpgtext.h"
+#include "lcf/rpg/saveeasyrpgwindow.h"
 
 namespace lcf {
 
@@ -57,6 +63,14 @@ struct TypeCategory {
 	static const Category::Index value = Category::Struct;
 };
 
+// Do not forget to add new Flags here
+template <> struct TypeCategory<rpg::TroopPageCondition::Flags>	{ static const Category::Index value = Category::Flags; };
+template <> struct TypeCategory<rpg::EventPageCondition::Flags>	{ static const Category::Index value = Category::Flags; };
+template <> struct TypeCategory<rpg::Terrain::Flags>			{ static const Category::Index value = Category::Flags; };
+template <> struct TypeCategory<rpg::SavePicture::Flags>		{ static const Category::Index value = Category::Flags; };
+template <> struct TypeCategory<rpg::SaveEasyRpgText::Flags>	{ static const Category::Index value = Category::Flags; };
+template <> struct TypeCategory<rpg::SaveEasyRpgWindow::Flags>	{ static const Category::Index value = Category::Flags; };
+
 template <> struct TypeCategory<rpg::Equipment>					{ static const Category::Index value = Category::RawStruct; };
 template <> struct TypeCategory<rpg::EventCommand>				{ static const Category::Index value = Category::RawStruct; };
 template <> struct TypeCategory<rpg::MoveCommand>				{ static const Category::Index value = Category::RawStruct; };
@@ -170,7 +184,7 @@ struct Primitive {
 		// FIXME: Bug #174
 		if (length != LcfSizeT<T>::value) {
 			dif = length - LcfSizeT<T>::value;
-			Log::Warning("Reading Primitive of incorrect size %" PRIu32 " (expected %" PRIu32 ") at %" PRIX32 "",
+			fprintf(stderr, "Reading Primitive of incorrect size %" PRIu32 " (expected %" PRIu32 ") at %" PRIX32 "\n",
 				length, LcfSizeT<T>::value, stream.Tell());
 		}
 
@@ -181,7 +195,9 @@ struct Primitive {
 
 		if (dif != 0) {
 			// Fix incorrect read pointer position
-			Log::Warning("Invalid Primitive at %" PRIX32 "", stream.Tell());
+#ifdef LCF_DEBUG_TRACE
+			printf("Invalid %s at %X\n", typeid(T).name(), stream.Tell());
+#endif
 			stream.Seek(dif, LcfReader::FromCurrent);
 		}
 	}
@@ -221,11 +237,11 @@ struct Primitive<std::vector<T>> {
 		stream.Read(ref, length);
 #ifdef LCF_DEBUG_TRACE
 		typename std::vector<T>::iterator it;
-		fprintf(stderr, "  ");
+		printf("  ");
 		for (it = ref.begin(); it != ref.end(); ++it) {
-			fprintf(stderr, "%d, ", static_cast<int>(*it));
+			printf("%d, ", static_cast<int>(*it));
 		}
-		fprintf(stderr, "\n");
+		printf("\n");
 #endif
 	}
 	static void WriteLcf(const std::vector<T>& ref, LcfWriter& stream) {
@@ -251,18 +267,16 @@ struct Primitive<int32_t> {
 		if (length >= 1 && length <= 5) {
 			ref = stream.ReadInt();
 #ifdef LCF_DEBUG_TRACE
-			fprintf(stderr, "  %d\n", ref);
-#endif
-		} else if (length == 0) {
-			ref = 0;
-#ifdef LCF_DEBUG_TRACE
-			fprintf(stderr, "  %d\n", ref);
+			printf("  %d\n", ref);
 #endif
 		} else {
 			ref = 0;
-			Log::Warning("Invalid integer at %X", stream.Tell());
+#ifdef LCF_DEBUG_TRACE
+			printf("Invalid integer at %X\n", stream.Tell());
+#endif
 			stream.Seek(length, LcfReader::FromCurrent);
 		}
+
 	}
 	static void WriteLcf(const int32_t& ref, LcfWriter& stream) {
 		stream.WriteInt(ref);
@@ -286,7 +300,7 @@ struct Primitive<std::string> {
 	static void ReadLcf(std::string& ref, LcfReader& stream, uint32_t length) {
 		stream.ReadString(ref, length);
 #ifdef LCF_DEBUG_TRACE
-		fprintf(stderr, "  %s\n", ref.c_str());
+		printf("  %s\n", ref.c_str());
 #endif
 	}
 	static void WriteLcf(const std::string& ref, LcfWriter& stream) {
@@ -311,11 +325,11 @@ struct Primitive<DBBitArray> {
 	static void ReadLcf(DBBitArray& ref, LcfReader& stream, uint32_t length) {
 		stream.ReadBits(ref, length);
 #ifdef LCF_DEBUG_TRACE
-		fprintf(stderr, "  ");
+		printf("  ");
 		for (auto& b: ref) {
-			fprintf(stderr, "%d", static_cast<int>(b));
+			print("%d", static_cast<int>(b));
 		}
-		fprintf(stderr, "\n");
+		printf("\n");
 #endif
 	}
 	static void WriteLcf(const DBBitArray& ref, LcfWriter& stream) {
@@ -759,7 +773,7 @@ public:
 
 	void StartElement(XmlReader& stream, const char* name, const char** /* atts */) {
 		if (strcmp(name, this->name) != 0)
-			Log::Error("XML: Expecting %s but got %s", this->name, name);
+			stream.Error("Expecting %s but got %s", this->name, name);
 		stream.SetHandler(handler);
 	}
 
@@ -779,7 +793,7 @@ public:
 
 	void StartElement(XmlReader& stream, const char* name, const char** /* atts */) {
 		if (strcmp(name, this->name) != 0)
-			Log::Error("XML: Expecting %s but got %s", this->name, name);
+			stream.Error("Expecting %s but got %s", this->name, name);
 		TypeReader<S>::BeginXml(ref, stream);
 	}
 
@@ -791,6 +805,4 @@ private:
 
 } //namespace lcf
 
-#include "fwd_flags_impl.h"
-
 #endif
